#include-once
#include <GuiComboBox.au3>
; #INDEX# =======================================================================================================================
; Title .........: ComboBox Auto-Fill
; AutoIt Version : 3.3.8.1
; Description ...: Automatically fill a ComboBox according to the text typed in it
; Author(s) .....: Matwachich
; ===============================================================================================================================

; #VARIABLES# ===================================================================================================================
Enum _
	$__gComboAutoFill_iActive, _
	$__gComboAutoFill_hParentWindow, _
	$__gComboAutoFill_hCombo, _
	$__gComboAutoFill_iMinVisible, _
	$__gComboAutoFill_sFillFunc, _
	$__gComboAutoFill_vUserData
Global $__gComboAutoFill_aGlobal[1][6] = [[0, "", "", "", "", ""]]
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;_ComboAutoFill_Create
;_ComboAutoFill_Fill
;_ComboAutoFill_SetActive
;_ComboAutoFill_SetPartialSearch
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================

; #FUNCTION# ====================================================================================================================
; Name ..........: _ComboAutoFill_Create
; Description ...: Set a ComboBox to be dynamically filled on text typing.
; Syntax ........: _ComboAutoFill_Create($hGui, $iComboBox, $sFillFunc[, $iMinVisible = 15[, $vUserData = 0]])
; Parameters ....: $hGui                - Parent GUI of the ComboBox Control.
;                  $iComboBox           - Control ID/Handle of the ComboBox.
;                  $sFillFunc           - Function called when a text is typed in the ComboBox (see Remarks).
;                  $iMinVisible         - [optional] Maximum visible items at once (see _GUICtrlComboBox_GetMinVisible). Default is 15.
;                  $vUserData           - [optional] Data passed to the $sFillFunc. Default is 0.
; Return values .: ID of the new created AutoFill-ComboBox (ComboBoxID)
; Author ........: Matwachich
; Remarks .......: The $sFillFunc must take 3 parameters: ComboBoxID, Current text in the ComboBox, User Data.
;                  |And it must return a 1-Based 1-Dimensional array of the items to fill in the ComboBox.
;                  |When this function is called, the ComboBox is filled with the result of a call to $sFillFunc, with
;                  |the second parameter (searched text) as an Empty String
; ===============================================================================================================================
Func _ComboAutoFill_Create($hGui, $iComboBox, $sFillFunc, $iMinVisible = 15, $vUserData = 0)
	ReDim $__gComboAutoFill_aGlobal[$__gComboAutoFill_aGlobal[0][0] + 2][6]
	$__gComboAutoFill_aGlobal[0][0] += 1
	Local $iIndex = $__gComboAutoFill_aGlobal[0][0]
	; ---
	If Not IsHWnd($iComboBox) Then $iComboBox = GUICtrlGetHandle($iComboBox)
	; ---
	If $iMinVisible < 1 Then $iMinVisible = 1
	_GUICtrlComboBox_SetMinVisible($iComboBox, $iMinVisible)
	; ---
	$__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_iActive] = 1
	$__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hParentWindow] = $hGui
	$__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hCombo] = $iComboBox
	$__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_iMinVisible] = _GUICtrlComboBox_GetMinVisible($iComboBox)
	$__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_sFillFunc] = $sFillFunc
	$__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_vUserData] = $vUserData
	; ---
	GUIRegisterMsg(0x0111, '__ComboAutoFill_WM_COMMAND')
	; ---
	__ComboAutoFill_Fill($iIndex, "")
	Return $iIndex
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _ComboAutoFill_Fill
; Description ...: Fill a ComboBox with the result of a call to it's Fill Func, with $sSearch as 2nd parameter.
; Syntax ........: _ComboAutoFill_Fill($iComboBox[, $sSearch = ""])
; Parameters ....: $iComboBox           - ComboBoxID (as returned by _ComboAutoFill_Create).
;                  $sSearch             - [optional] String to pass to the Fill Func. Default is "".
; Return values .: Success - 1
;                  Failure - 0 And set @error = 1 if the $iComboBox is not valid
; Author ........: Matwachich
; ===============================================================================================================================
Func _ComboAutoFill_Fill($iComboBox, $sSearch = "")
	If Not __ComboAutoFill_ComboIDIsValid($iComboBox) Then Return SetError(1, 0, 0)
	; ---
	__ComboAutoFill_Fill($iComboBox, $sSearch)
	Return 1
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _ComboAutoFill_SetActive
; Description ...: Activate/Deactivate the Auto-Fill system of a ComboBox.
; Syntax ........: _ComboAutoFill_SetActive($iComboBox, $iActive)
; Parameters ....: $iComboBox           - ComboBoxID (as returned by _ComboAutoFill_Create).
;                  $iActive             - 1 to activate, 0 to deactivate.
; Return values .: Success - 1
;                  Failure - 0 And set @error = 1 if the $iComboBox is not valid
; Author ........: Matwachich
; Remarks .......: Each time you call this function, the ComboBox is filled with the result of a call to $sFillFunc, with
;                  |the second parameter (searched text) as an Empty String
; ===============================================================================================================================
Func _ComboAutoFill_SetActive($iComboBox, $iActive)
	If Not __ComboAutoFill_ComboIDIsValid($iComboBox) Then Return SetError(1, 0, 0)
	; ---
	$__gComboAutoFill_aGlobal[$iComboBox][$__gComboAutoFill_iActive] = $iActive
	__ComboAutoFill_Fill($iComboBox, "")
	Return 1
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _ComboAutoFill_SetUserData
; Description ...: Set user data associated with a ComboBox.
; Syntax ........: _ComboAutoFill_SetUserData($iComboBox, $iPartial)
; Parameters ....: $iComboBox           - ComboBoxID (as returned by _ComboAutoFill_Create).
;                  $vUserData           - New User Data.
; Return values .: Success - 1
;                  Failure - 0 And set @error = 1 if the $iComboBox is not valid
; Author ........: Matwachich
; ===============================================================================================================================
Func _ComboAutoFill_SetUserData($iComboBox, $iPartial)
	If Not __ComboAutoFill_ComboIDIsValid($iComboBox) Then Return SetError(1, 0, 0)
	; ---
	$__gComboAutoFill_aGlobal[$iComboBox][$__gComboAutoFill_vUserData] = $iPartial
	Return 1
EndFunc

; #INTERNAL_USE_ONLY# ===========================================================================================================

Func __ComboAutoFill_OnChange($iIndex, $sText)
	If $sText And __ComboAutoFill_Fill($iIndex, $sText) Then
		_GUICtrlComboBox_ShowDropDown($__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hCombo], True)
	Else
		_GUICtrlComboBox_ShowDropDown($__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hCombo], False)
	EndIf
EndFunc

Func __ComboAutoFill_Fill($iIndex, $sText)
	Local $aData = Call($__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_sFillFunc], _
							$iIndex, $sText, $__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_vUserData])
	; ---
	_GUICtrlComboBox_BeginUpdate($__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hCombo])
	_GUICtrlComboBox_ResetContent($__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hCombo])
	; ---
	If IsArray($aData) Then
		; ---
		; Fill
		For $i = 1 To $aData[0]
			_GUICtrlComboBox_AddString($__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hCombo], $aData[$i])
		Next
		; ---
		; Resize
		If $aData[0] < $__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_iMinVisible] Then
			_GUICtrlComboBox_SetMinVisible($__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hCombo], $aData[0])
		Else
			_GUICtrlComboBox_SetMinVisible($__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hCombo], $__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_iMinVisible])
		EndIf
	EndIf
	; ---
	_GUICtrlComboBox_EndUpdate($__gComboAutoFill_aGlobal[$iIndex][$__gComboAutoFill_hCombo])
	; ---
	If IsArray($aData) And $aData[0] > 0 Then Return 1
	Return 0
EndFunc

; http://www.autoitscript.com/forum/topic/47709-combobox-onchange-event-not-triggered-after-typing/#entry357491
Func __ComboAutoFill_WM_COMMAND($hWnd, $msgID, $wParam, $lParam)
	Local $nNotifyCode = BitShift($wParam, 16) ; HiWord
	; ---
	For $i = 1 To $__gComboAutoFill_aGlobal[0][0]
		If Not $__gComboAutoFill_aGlobal[$i][$__gComboAutoFill_iActive] Then ContinueLoop
		; ---
		If $__gComboAutoFill_aGlobal[$i][$__gComboAutoFill_hCombo] = $lParam Then
			Switch $nNotifyCode
				Case $CBN_EDITUPDATE ; when user types in new data
					Local $sData = _GUICtrlComboBox_GetEditText($__gComboAutoFill_aGlobal[$i][$__gComboAutoFill_hCombo])
					__ComboAutoFill_OnChange($i, $sData)
					_GUICtrlComboBox_SetEditText($__gComboAutoFill_aGlobal[$i][$__gComboAutoFill_hCombo], $sData)
					Return "GUI_RUNDEFMSG"
			EndSwitch
		EndIf
	Next
	; ---
    Return "GUI_RUNDEFMSG"
EndFunc

Func __ComboAutoFill_ComboIDIsValid($iComboBox)
	If $iComboBox < 0 Or $iComboBox > $__gComboAutoFill_aGlobal[0][0] Then Return 0
	Return 1
EndFunc

Func __ComboAutoFill_hCtrlToID($hCtrl)
	For $i = 1 To $__gComboAutoFill_aGlobal[0][0]
		If $hCtrl = $__gComboAutoFill_aGlobal[$i][$__gComboAutoFill_hCombo] Then Return $i
	Next
	Return -1
EndFunc
